/*
 *  Licensed to the author under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.java.quickcheck.srcgenerator;

import static java.lang.String.*;

final class Parameter {
	String name;
	String type;

	Parameter(CharSequence name, CharSequence type) {
		Assertion.assertNotNull(name, "name");
		Assertion.assertNotNull(type, "type");
		this.name = name.toString();
		this.type = type.toString();
	}
	
	@Override public boolean equals(Object obj){
		if(!(obj instanceof Parameter)){
			return false;
		}
		Parameter other = (Parameter) obj;
		return name.equals(other.name) && type.equals(other.type);
	}
	
	@Override public String toString() { 
		return format("Parameter[name=%s, type=%s]", name, type);
	}
	
	@Override public int hashCode() {
		return name.hashCode(); 
	}
}
